class NameConst:
    kSessionID = "SessionID"

    kAction   = "Action"    # noqa
    vOpen     = "Open"      # noqa
    vClose    = "Close"     # noqa
    vSend     = "Send"      # noqa
    vExternal = "External"
    vResetSeq = "ResetSeq"

    kShareType    = "ShareType"  # noqa
    vSharedMemory = 1
    vSocket       = 2            # noqa

    kSharedMemoryName = "SharedMemoryName"

    kRange = "Range"
    kSeq   = "Seq"    # noqa
    kTime  = "Time"   # noqa
    kBody  = "Body"   # noqa

    kErr         = "Err"        # noqa
    vErrOK       = "OK"         # noqa
    vErrNoMemory = "No memory"
